@@ -7,7 +7,7 @@
    will, as long as this notice is kept intact and this source code is made
    available. There is no warranty, express or implied. */
 
-#ifdef HAVE_CONFIG_H
+#if HAVE_CONFIG_H
 # include <config.h>
 #elif !defined(__cplusplus)
 /* Assume we don't have inline by default */
